package cn.springmvc.util;
import org.apache.commons.collections.CollectionUtils;
import org.apache.ibatis.cache.CacheKey;
import org.apache.ibatis.executor.Executor;
import org.apache.ibatis.mapping.BoundSql;
import org.apache.ibatis.mapping.MappedStatement;
import org.apache.ibatis.mapping.ParameterMapping;
import org.apache.ibatis.plugin.*;
import org.apache.ibatis.reflection.MetaObject;
import org.apache.ibatis.session.Configuration;
import org.apache.ibatis.session.ResultHandler;
import org.apache.ibatis.session.RowBounds;
import org.apache.ibatis.type.TypeHandlerRegistry;
 
 
import java.text.DateFormat;
import java.util.Date;
import java.util.List;
import java.util.Locale;
import java.util.Properties;
import java.util.regex.Matcher;

@Intercepts(value = {  
        @Signature(type = Executor.class,  
                method = "update",  
                args = {MappedStatement.class, Object.class}),  
        @Signature(type = Executor.class,  
                method = "query",  
                args = {MappedStatement.class, Object.class, RowBounds.class, ResultHandler.class,  
                        CacheKey.class, BoundSql.class}),  
        @Signature(type = Executor.class,  
                method = "query",  
                args = {MappedStatement.class, Object.class, RowBounds.class, ResultHandler.class})})  
public class MybatisInterceptor implements Interceptor {
 
    @Override
    public Object intercept(Invocation invocation) throws Throwable {
        try{
            MappedStatement mappedStatement = (MappedStatement) invocation.getArgs()[0];  // ȡxmlеһselect/update/insert/deleteڵ㣬ҪһSQL
            Object parameter = null;
            if (invocation.getArgs().length > 1) {
                parameter = invocation.getArgs()[1];
                System.out.println("parameter = " + parameter);
            }
            String sqlId = mappedStatement.getId(); // ȡڵid,sqlid
            BoundSql boundSql = mappedStatement.getBoundSql(parameter);  // BoundSqlǷװmyBatisղsql
            Configuration configuration = mappedStatement.getConfiguration();  // ȡڵ
            String sql = getSql(configuration, boundSql, sqlId); // ȡյsql
            System.err.println("sql = " + sql);
            //log.debug(sql);
        }catch(Exception e){
           // log.error(e.getMessage(), e);
        }
        return invocation.proceed();  // ִ󣬲ıԭеsqlִй
    }
    public static String getSql(Configuration configuration, BoundSql boundSql,String sqlId) {
        String sql = showSql(configuration, boundSql);
        StringBuilder str = new StringBuilder(100);
        str.append(sqlId);
        str.append(":");
        str.append(sql);
        return str.toString();
    }
    private static String getParameterValue(Object obj) {
        String value = null;
        if (obj instanceof String) {
            value = "'" + obj.toString() + "'";
        } else if (obj instanceof Date) {
            DateFormat formatter = DateFormat.getDateTimeInstance(DateFormat.DEFAULT, DateFormat.DEFAULT, Locale.CHINA);
            value = "'" + formatter.format(new Date()) + "'";
        } else {
            if (obj != null) {
                value = obj.toString();
            } else {
                value = "";
            }
 
        }
        return value;
    }
    public static String showSql(Configuration configuration, BoundSql boundSql) {
        Object parameterObject = boundSql.getParameterObject();  // ȡ
        List<ParameterMapping> parameterMappings = boundSql
                .getParameterMappings();
        String sql = boundSql.getSql().replaceAll("[\\s]+", " ");  // sqlжոһո
        if (CollectionUtils.isNotEmpty(parameterMappings) && parameterObject != null) {
            TypeHandlerRegistry typeHandlerRegistry = configuration.getTypeHandlerRegistry(); // ȡʹעʹĹǽjavaͺݿ͵ת<br>// parameterObject.getClass(ҵӦͣ滻
            if (typeHandlerRegistry.hasTypeHandler(parameterObject.getClass())) {
                sql = sql.replaceFirst("\\?", Matcher.quoteReplacement(getParameterValue(parameterObject)));
 
            } else {
                MetaObject metaObject = configuration.newMetaObject(parameterObject);// MetaObjectҪǷװoriginalObjectṩgetsetķڻȡoriginalObjectֵ,Ҫֶ֧JavaBeanCollectionMapͶĲ
                for (ParameterMapping parameterMapping : parameterMappings) {
                    String propertyName = parameterMapping.getProperty();
                    if (metaObject.hasGetter(propertyName)) {
                        Object obj = metaObject.getValue(propertyName);
                        sql = sql.replaceFirst("\\?", Matcher.quoteReplacement(getParameterValue(obj)));
                    } else if (boundSql.hasAdditionalParameter(propertyName)) {
                        Object obj = boundSql.getAdditionalParameter(propertyName);  // ÷֧Ƕ̬sql
                        sql = sql.replaceFirst("\\?", Matcher.quoteReplacement(getParameterValue(obj)));
                    }else{sql=sql.replaceFirst("\\?","ȱʧ");}//ӡȱʧѸòȱʧֹλ
                }
            }
        }
        return sql;
    }
 
    @Override
    public Object plugin(Object target) {
        return Plugin.wrap(target, this);
    }
 
    @Override
    public void setProperties(Properties properties) {
       
    }
}